import logging
from multiprocessing import Process
from rockettm import tasks
import traceback
import pika
import json
import sys
import os
from timekiller import call
import importlib


if len(sys.argv) == 2:
    i, f = os.path.split(sys.argv[1])
    sys.path.append(i)
    settings = __import__(os.path.splitext(f)[0])
else:
    sys.path.append(os.getcwd())
    try:
        import settings
    except:
        exit("settings.py not found")
try:
    logging.basicConfig(**settings.logger)
except:
    pass

for mod in settings.imports:
    importlib.import_module(mod)


def worker(name, concurrency, durable=False, max_time=-1):
    def callback(ch, method, properties, body):
        # py3 support
        if isinstance(body, bytes):
            body = body.decode('utf-8')

        recv = json.loads(body)
        logging.info("execute %s" % recv['event'])
        try:
            print("recv", recv)
            for func, max_time2 in tasks.subs[recv['event']]:
                if max_time2 != -1:
                    apply_max_time = max_time2
                else:
                    apply_max_time = max_time
                call(func, apply_max_time, *recv['args'])
        except:
            logging.error(traceback.format_exc())
        finally:
            ch.basic_ack(delivery_tag=method.delivery_tag)

    conn = pika.BlockingConnection(pika.ConnectionParameters(settings.ip))
    channel = conn.channel()
    logging.info("create queue: %s durable: %s" % (name, durable))
    channel.queue_declare(queue=name, durable=durable)
    channel.basic_qos(prefetch_count=1)
    channel.basic_consume(callback, queue=name, no_ack=False)
    channel.start_consuming()


def main():
    for queue in settings.queues:
        for x in range(queue['concurrency']):
            p = Process(target=worker, kwargs=queue)
            logging.info("start process worker: %s queue: %s" % (worker,
                                                                 queue))
            p.start()

if __name__ == "__main__":
    main()



from pika import BlockingConnection, ConnectionParameters
import json
import logging


class tasks(object):
    subs = {}
    queues = []
    ip = "localhost"
    conn = False
    channel = False

    @staticmethod
    def connect(ip=None):
        logging.info("rabbitmq connect %s" % tasks.ip)
        if ip:
            tasks.ip = ip
        tasks.conn = BlockingConnection(ConnectionParameters(tasks.ip))
        tasks.channel = tasks.conn.channel()

    @staticmethod
    def add_task(event, func, max_time=-1):
        logging.info("add task %s" % event)
        if event not in tasks.subs:
            tasks.subs[event] = []
        tasks.subs[event].append((func, max_time))

    @staticmethod
    def task(event, max_time=-1):
        def wrap_function(func):
            tasks.add_task(event, func, max_time)
            return func
        return wrap_function

    @staticmethod
    def send_task(queue, event, *args):
        logging.info("send task to queue %s, event %s" % (queue, event))
        if not tasks.channel or tasks.channel.is_closed:
            tasks.connect()
        if queue not in tasks.queues:
            try:
                tasks.channel.queue_declare(queue=queue, passive=True)
                tasks.queues.append(queue)
            except:
                error = "Queue not declare, first start the server"
                logging.error(error)
                raise Exception(error)

        tasks.channel.basic_publish(exchange='',
                                    routing_key=queue,
                                    body=json.dumps({'event': event,
                                                     'args': args}))

# avoids having to import tasks
connect = tasks.connect
send_task = tasks.send_task
add_task = tasks.add_task
task = tasks.task
